@@ -11,10 +11,15 @@
  *
  * Available variables:
  * - $node: The node object for this webform.
- * - $confirmation_message: The confirmation message input by the webform author.
+ * - $progressbar: The progress bar 100% filled (if configured). This may not
+ *   print out anything if a progress bar is not enabled for this node.
+ * - $confirmation_message: The confirmation message input by the webform
+ *   author.
  * - $sid: The unique submission ID of this submission.
+ * - $url: The URL of the form (or for in-block confirmations, the same page).
  */
 ?>
+<?php print $progressbar; ?>
 
 <div class="webform-confirmation">
   <?php if ($confirmation_message): ?>
@@ -25,5 +30,5 @@
 </div>
 
 <div class="links">
-  <a href="<?php print url('node/'. $node->nid) ?>"><?php print t('Go back to the form') ?></a>
+  <a href="<?php print $url; ?>"><?php print t('Go back to the form'); ?></a>
 </div>
