@@ -24,12 +24,13 @@
  * @see webform_options_example()
  * @see hook_webform_select_options_info_alter()
  *
- * @return
+ * @return array
  *   An array of callbacks that can be used for select list options. This array
  *   should be keyed by the "name" of the pre-defined list. The values should
  *   be an array with the following additional keys:
  *     - title: The translated title for this list.
- *     - options callback: The name of the function that will return the list.
+ *     - options callback: The name of a function implementing
+ *       callback_webform_options() that will return the list.
  *     - options arguments: Any additional arguments to send to the callback.
  *     - file: Optional. The file containing the options callback, relative to
  *       the module root.
@@ -49,7 +50,7 @@ function hook_webform_select_options_info() {
 /**
  * Alter the list of select list options provided by Webform and other modules.
  *
- * @see hook_webform_select_options_info().
+ * @see hook_webform_select_options_info()
  */
 function hook_webform_select_options_info_alter(&$items) {
   // Remove the days of the week options.
@@ -57,12 +58,9 @@ function hook_webform_select_options_info_alter(&$items) {
 }
 
 /**
- * This is an example function to demonstrate a webform options callback.
+ * Define a list of options that Webform may use in a select component.
  *
- * This function returns a list of options that Webform may use in a select
- * component. In order to be called, the function name
- * ("webform_options_example" in this case), needs to be specified as a callback
- * in hook_webform_select_options_info().
+ * Callback for hook_webform_select_options_info().
  *
  * @param $component
  *   The Webform component array for the select component being displayed.
@@ -71,17 +69,14 @@ function hook_webform_select_options_info_alter(&$items) {
  *   of key => value pairs. Select components support up to one level of
  *   nesting, but when results are displayed, the list needs to be returned
  *   without the nesting.
- * @param $filter
- *   Boolean value indicating whether the included options should be passed
- *   through the _webform_filter_values() function for token replacement (only)
- *   needed if your list contains tokens).
  * @param $arguments
  *   The "options arguments" specified in hook_webform_select_options_info().
- * @return
+ *
+ * @return array
  *   An array of key => value pairs suitable for a select list's #options
  *   FormAPI property.
  */
-function webform_options_example($component, $flat, $filter, $arguments) {
+function callback_webform_options($component, $flat, $arguments) {
   $options = array(
     'one' => t('Pre-built option one'),
     'two' => t('Pre-built option two'),
@@ -104,6 +99,31 @@ function hook_webform_submission_load(&$submissions) {
   }
 }
 
+/**
+ * Respond to the creation of a new submission from form values.
+ *
+ * This hook is called when a user has completed a submission to initialize the
+ * submission object. After this object has its values populated, it will be
+ * saved by webform_submission_insert(). Note that this hook is only called for
+ * new submissions, not for submissions being edited. If responding to the
+ * saving of all submissions, it's recommended to use
+ * hook_webform_submission_presave().
+ *
+ * @param $submission
+ *   The submission object that has been created.
+ * @param $node
+ *   The Webform node for which this submission is being saved.
+ * @param $account
+ *   The user account that is creating the submission.
+ * @param $form_state
+ *   The contents of form state that is the basis for this submission.
+ *
+ * @see webform_submission_create()
+ */
+function hook_webform_submission_create_alter(&$submission, &$node, &$account, &$form_state) {
+  $submission->new_property = TRUE;
+}
+
 /**
  * Modify a Webform submission, prior to saving it in the database.
  *
@@ -115,7 +135,7 @@ function hook_webform_submission_load(&$submissions) {
 function hook_webform_submission_presave($node, &$submission) {
   // Update some component's value before it is saved.
   $component_id = 4;
-  $submission->data[$component_id]['value'][0] = 'foo';
+  $submission->data[$component_id][0] = 'foo';
 }
 
 /**
@@ -193,8 +213,13 @@ function hook_webform_submission_delete($node, $submission) {
  *   The Webform node on which this submission was made.
  * @param $submission
  *   The Webform submission on which the actions may be performed.
+ *
+ * @return array
+ *   List of action.
  */
 function hook_webform_submission_actions($node, $submission) {
+  $actions = array();
+
   if (webform_results_access($node)) {
     $actions['myaction'] = array(
       'title' => t('Do my action'),
@@ -206,11 +231,32 @@ function hook_webform_submission_actions($node, $submission) {
   return $actions;
 }
 
+/**
+ * Modify the draft to be presented for editing.
+ *
+ * When drafts are enabled for the webform, by default, a pre-existing draft is
+ * presented when the webform is displayed to that user. To allow multiple
+ * drafts, implement this alter function to set the $sid to NULL, or use your
+ * application's business logic to determine whether a new draft or which of
+ * he pre-existing drafts should be presented.
+ *
+ * @param int $sid
+ *   The id of the most recent submission to be presented for editing. Change
+ *    to a different draft's sid or set to NULL for a new draft.
+ * @param array $context
+ *   Array of context with indices 'nid' and 'uid'.
+ */
+function hook_webform_draft_alter(&$sid, $context) {
+  if ($_GET['newdraft']) {
+    $sid = NULL;
+  }
+}
+
 /**
  * Alter the display of a Webform submission.
  *
  * This function applies to both e-mails sent by Webform and normal display of
- * submissions when viewing through the adminsitrative interface.
+ * submissions when viewing through the administrative interface.
  *
  * @param $renderable
  *   The Webform submission in a renderable array, similar to FormAPI's
@@ -301,11 +347,70 @@ function hook_webform_component_delete($component) {
     ->execute();
 }
 
+/**
+ * Alter the entire analysis before rendering to the page on the Analysis tab.
+ *
+ * This alter hook allows modification of the entire analysis of a node's
+ * Webform results. The resulting analysis is displayed on the Results ->
+ * Analysis tab on the Webform.
+ *
+ * @param array $analysis
+ *   A Drupal renderable array, passed by reference, containing the entire
+ *   contents of the analysis page. This typically will contain the following
+ *   two major keys:
+ *   - form: The form for configuring the shown analysis.
+ *   - components: The list of analyses for each analysis-enabled component
+ *     for the node. Each keyed by its component ID.
+ */
+function hook_webform_analysis_alter(&$analysis) {
+  $node = $analysis['#node'];
+
+  // Add an additional piece of information to every component's analysis:
+  foreach (element_children($analysis['components']) as $cid) {
+    $component = $node->components[$cid];
+    $analysis['components'][$cid]['chart'] = array(
+      '#markup' => t('Chart for the @name component', array('@name' => $component['name'])),
+    );
+  }
+}
+
+/**
+ * Alter data when displaying an analysis on that component.
+ *
+ * This hook modifies the data from an individual component's analysis results.
+ * It can be used to add additional analysis, or to modify the existing results.
+ * If needing to alter the entire set of analyses rather than an individual
+ * component, hook_webform_analysis_alter() may be used instead.
+ *
+ * @param array $data
+ *   An array containing the result of a components analysis hook, passed by
+ *   reference. This is passed directly from a component's
+ *   _webform_analysis_component() function. See that hook for more information
+ *   on this value.
+ * @param object $node
+ *   The node object that contains the component being analyzed.
+ * @param array $component
+ *   The Webform component array whose analysis results are being displayed.
+ *
+ * @see _webform_analysis_component()
+ * @see hook_webform_analysis_alter()
+ */
+function hook_webform_analysis_component_data_alter(&$data, $node, $component) {
+  if ($component['type'] === 'textfield') {
+    // Do not display rows that contain a zero value.
+    foreach ($data as $row_number => $row_data) {
+      if ($row_data[1] === 0) {
+        unset($data[$row_number]);
+      }
+    }
+  }
+}
+
 /**
  * Alter a Webform submission's header when exported.
  */
 function hook_webform_csv_header_alter(&$header, $component) {
-  // Use the machine name for component headers, but only for the webform 
+  // Use the machine name for component headers, but only for the webform
   // with node 5 and components that are text fields.
   if ($component['nid'] == 5 && $component['type'] == 'textfield') {
     $header[2] = $component['form_key'];
@@ -326,7 +431,7 @@ function hook_webform_csv_data_alter(&$data, $component, $submission) {
 /**
  * Define components to Webform.
  *
- * @return
+ * @return array
  *   An array of components, keyed by machine name. Required properties are
  *   "label" and "description". The "features" array defines which capabilities
  *   the component has, such as being displayed in e-mails or csv downloads.
@@ -371,8 +476,8 @@ function hook_webform_csv_data_alter(&$data, $component, $submission) {
  *     - csv_headers
  *     - csv_data
  *
- * See the sample component implementation for details on each one of these
- * callbacks.
+ *   See the sample component implementation for details on each one of these
+ *   callbacks.
  *
  * @see webform_components()
  */
@@ -383,6 +488,9 @@ function hook_webform_component_info() {
     'label' => t('Textfield'),
     'description' => t('Basic textfield type.'),
     'features' => array(
+      // This component includes an analysis callback. Defaults to TRUE.
+      'analysis' => TRUE,
+
       // Add content to CSV downloads. Defaults to TRUE.
       'csv' => TRUE,
 
@@ -430,7 +538,17 @@ function hook_webform_component_info() {
       // If this component saves a file that can be used as an e-mail
       // attachment. Defaults to FALSE.
       'attachment' => FALSE,
+
+      // If this component reflects a time range and should use labels such as
+      // "Before" and "After" when exposed as filters in Views module.
+      'views_range' => FALSE,
     ),
+
+    // Specify the conditional behaviour of this component.
+    // Examples are 'string', 'date', 'time', 'numeric', 'select'.
+    // Defaults to 'string'.
+    'conditional_type' => 'string',
+
     'file' => 'components/textfield.inc',
   );
 
@@ -498,7 +616,8 @@ function hook_webform_component_defaults_alter(&$defaults, $type) {
  *   - "list"
  * @param $account
  *   A user account object.
- * @return
+ *
+ * @return bool
  *   TRUE if the current user has access to submission,
  *   or FALSE otherwise.
  */
@@ -506,16 +625,20 @@ function hook_webform_submission_access($node, $submission, $op = 'view', $accou
   switch ($op) {
     case 'view':
       return TRUE;
-      break;
+
+    break;
     case 'edit':
       return FALSE;
-      break;
+
+    break;
     case 'delete':
       return TRUE;
-      break;
+
+    break;
     case 'list':
       return TRUE;
-      break;
+
+    break;
   }
 }
 
@@ -524,14 +647,17 @@ function hook_webform_submission_access($node, $submission, $op = 'view', $accou
  *
  * Note in addition to the view access to the results granted here, the $account
  * must also have view access to the Webform node in order to see results.
+ * Access via this hook is in addition (adds permission) to the standard
+ * webform access.
  *
- * @see webform_results_access().
+ * @see webform_results_access()
  *
  * @param $node
  *   The Webform node to check access on.
  * @param $account
  *   The user account to check access on.
- * @return
+ *
+ * @return bool
  *   TRUE or FALSE if the user can access the webform results.
  */
 function hook_webform_results_access($node, $account) {
@@ -544,6 +670,62 @@ function hook_webform_results_access($node, $account) {
   }
 }
 
+/**
+ * Determine if a user has access to clear the results of a webform.
+ *
+ * Access via this hook is in addition (adds permission) to the standard
+ * webform access (delete all webform submissions).
+ *
+ * @see webform_results_clear_access()
+ *
+ * @param object $node
+ *   The Webform node to check access on.
+ * @param object $account
+ *   The user account to check access on.
+ *
+ * @return bool
+ *   TRUE or FALSE if the user can access the webform results.
+ */
+function hook_webform_results_clear_access($node, $account) {
+  return user_access('my additional access', $account);
+}
+
+/**
+ * Overrides the node_access and user_access permission to access and edit
+ * webform components, e-mails, conditions, and form settings.
+ *
+ * Return NULL to defer to other modules. If all implementations defer, then
+ * access to the node's EDIT tab plus 'edit webform components' permission
+ * determines access. To grant access, return TRUE; to deny access, return
+ * FALSE. If more than one implementation return TRUE/FALSE, all must be TRUE
+ * to grant access.
+ *
+ * In this way, access to the EDIT tab of the node may be decoupled from
+ * access to the WEBFORM tab. When returning TRUE, consider all aspects of
+ * access as this will be the only test. For example, 'return TRUE;' would grant
+ * annonymous access to creating webform components, which seldom be desired.
+ *
+ * @see webform_node_update_access()
+ *
+ * @param object $node
+ *   The Webform node to check access on.
+ * @param object $account
+ *   The user account to check access on.
+ *
+ * @return bool|null
+ *   TRUE or FALSE if the user can access the webform results, or NULL if
+ *   access should be deferred to other implementations of this hook or
+ *   node_access('update') plus user_access('edit webform components').
+ */
+function hook_webform_update_access($node, $account) {
+  // Allow anyone who can see webform_editable_by_user nodes and who has
+  // 'my webform component edit access' permission to see, edit, and delete the
+  // webform components, e-mails, conditionals, and form settings.
+  if ($node->type == 'webform_editable_by_user') {
+    return node_access('view', $node, $account) && user_access('my webform component edit access', $account);
+  }
+}
+
 /**
  * Return an array of files associated with the component.
  *
@@ -554,19 +736,21 @@ function hook_webform_results_access($node, $account) {
  * @param $value
  *   An array of information containing the submission result, directly
  *   correlating to the webform_submitted_data database schema.
- * @return
+ *
+ * @return array
  *   An array of files, each file is an array with following keys:
  *     - filepath: The relative path to the file.
  *     - filename: The name of the file including the extension.
  *     - filemime: The mimetype of the file.
  *   This will result in an array looking something like this:
- *   @code
+ *
+ * @code
  *   array[0] => array(
  *     'filepath' => '/sites/default/files/attachment.txt',
  *     'filename' => 'attachment.txt',
  *     'filemime' => 'text/plain',
  *   );
- *   @endcode
+ * @endcode
  */
 function _webform_attachments_component($component, $value) {
   $files = array();
@@ -574,6 +758,63 @@ function _webform_attachments_component($component, $value) {
   return $files;
 }
 
+/**
+ * Alter default settings for a newly created webform node.
+ *
+ * @param array $defaults
+ *   Default settings for a newly created webform node as defined by webform_node_defaults().
+ *
+ * @see webform_node_defaults()
+ */
+function hook_webform_node_defaults_alter(&$defaults) {
+  $defaults['allow_draft'] = '1';
+}
+
+/**
+ * Add additional fields to submission data downloads.
+ *
+ * @return array
+ *   Keys and titles for default submission information.
+ *
+ * @see hook_webform_results_download_submission_information_data()
+ */
+function hook_webform_results_download_submission_information_info() {
+  return array(
+    'field_key_1' => t('Field Title 1'),
+    'field_key_2' => t('Field Title 2'),
+  );
+}
+
+/**
+ * Return values for submission data download fields.
+ *
+ * @param $token
+ *   The name of the token being replaced.
+ * @param $submission
+ *   The data for an individual submission from webform_get_submissions().
+ * @param array $options
+ *   A list of options that define the output format. These are generally passed
+ *   through from the GUI interface.
+ * @param $serial_start
+ *   The starting position for the Serial column in the output.
+ * @param $row_count
+ *   The number of the row being generated.
+ *
+ * @return string
+ *   Value for requested submission information field.
+ *
+ * @see hook_webform_results_download_submission_information_info()
+ */
+function hook_webform_results_download_submission_information_data($token, $submission, array $options, $serial_start, $row_count) {
+  switch ($token) {
+    case 'field_key_1':
+      return 'Field Value 1';
+
+    case 'field_key_2':
+      return 'Field Value 2';
+  }
+}
+
 /**
  * @}
  */
@@ -590,14 +831,14 @@ function _webform_attachments_component($component, $value) {
 /**
  * Specify the default properties of a component.
  *
- * @return
+ * @return array
  *   An array defining the default structure of a component.
  */
 function _webform_defaults_component() {
   return array(
     'name' => '',
     'form_key' => NULL,
-    'mandatory' => 0,
+    'required' => 0,
     'pid' => 0,
     'weight' => 0,
     'extra' => array(
@@ -606,6 +847,9 @@ function _webform_defaults_component() {
       'optrand' => 0,
       'qrand' => 0,
       'description' => '',
+      'description_above' => FALSE,
+      'private' => FALSE,
+      'analysis' => TRUE,
     ),
   );
 }
@@ -621,7 +865,8 @@ function _webform_defaults_component() {
  *
  * @param $component
  *   A Webform component array.
- * @return
+ *
+ * @return array
  *   An array of form items to be displayed on the edit component page
  */
 function _webform_edit_component($component) {
@@ -636,7 +881,7 @@ function _webform_edit_component($component) {
     '#type' => 'textarea',
     '#title' => t('Options'),
     '#default_value' => $component['extra']['options'],
-    '#description' => t('Key-value pairs may be entered separated by pipes. i.e. safe_key|Some readable option') . theme('webform_token_help'),
+    '#description' => t('Key-value pairs may be entered separated by pipes. i.e. safe_key|Some readable option') . ' ' . theme('webform_token_help'),
     '#cols' => 60,
     '#rows' => 5,
     '#weight' => -3,
@@ -659,19 +904,25 @@ function _webform_edit_component($component) {
  *   Whether or not to filter the contents of descriptions and values when
  *   rendering the component. Values need to be unfiltered to be editable by
  *   Form Builder.
+ * @param $submission
+ *   The submission from which this component is being rendered. Usually not
+ *   needed. Used by _webform_render_date() to validate using the submission's
+ *   completion date.
+ *
+ * @return array
+ *   $form_item
  *
  * @see _webform_client_form_add_component()
  */
-function _webform_render_component($component, $value = NULL, $filter = TRUE) {
+function _webform_render_component($component, $value = NULL, $filter = TRUE, $submission = NULL) {
   $form_item = array(
     '#type' => 'textfield',
-    '#title' => $filter ? _webform_filter_xss($component['name']) : $component['name'],
-    '#required' => $component['mandatory'],
+    '#title' => $filter ? webform_filter_xss($component['name']) : $component['name'],
+    '#required' => $component['required'],
     '#weight' => $component['weight'],
-    '#description'   => $filter ? _webform_filter_descriptions($component['extra']['description']) : $component['extra']['description'],
-    '#default_value' => $filter ? _webform_filter_values($component['value']) : $component['value'],
-    '#prefix' => '<div class="webform-component-textfield" id="webform-component-' . $component['form_key'] . '">',
-    '#suffix' => '</div>',
+    '#description'   => $filter ? webform_filter_descriptions($component['extra']['description']) : $component['extra']['description'],
+    '#default_value' => $filter ? webform_replace_tokens($component['value']) : $component['value'],
+    '#theme_wrappers' => array('webform_element'),
   );
 
   if (isset($value)) {
@@ -681,6 +932,23 @@ function _webform_render_component($component, $value = NULL, $filter = TRUE) {
   return $form_item;
 }
 
+/**
+ * Allow modules to modify a webform component that is going to be rendered in a form.
+ *
+ * @param array $element
+ *   The display element as returned by _webform_render_component().
+ * @param array $component
+ *   A Webform component array.
+ *
+ * @see _webform_render_component()
+ */
+function hook_webform_component_render_alter(&$element, &$component) {
+  if ($component['cid'] == 10) {
+    $element['#title'] = 'My custom title';
+    $element['#default_value'] = 42;
+  }
+}
+
 /**
  * Display the result of a submission for a component.
  *
@@ -696,7 +964,10 @@ function _webform_render_component($component, $value = NULL, $filter = TRUE) {
  *   Either 'html' or 'text'. Defines the format that the content should be
  *   returned as. Make sure that returned content is run through check_plain()
  *   or other filtering functions when returning HTML.
- * @return
+ * @param $submission
+ *   The submission. Used to generate tokens.
+ *
+ * @return array
  *   A renderable element containing at the very least these properties:
  *    - #title
  *    - #weight
@@ -707,7 +978,7 @@ function _webform_render_component($component, $value = NULL, $filter = TRUE) {
  *   which will properly format the label and content for use within an e-mail
  *   (such as wrapping the text) or as HTML (ensuring consistent output).
  */
-function _webform_display_component($component, $value, $format = 'html') {
+function _webform_display_component($component, $value, $format = 'html', $submission = array()) {
   return array(
     '#title' => $component['name'],
     '#weight' => $component['weight'],
@@ -722,6 +993,44 @@ function _webform_display_component($component, $value, $format = 'html') {
   );
 }
 
+/**
+ * Allow modules to modify a "display only" webform component.
+ *
+ * @param array $element
+ *   The display element as returned by _webform_display_component().
+ * @param array $component
+ *   A Webform component array.
+ *
+ * @see _webform_display_component()
+ */
+function hook_webform_component_display_alter(&$element, &$component) {
+  if ($component['cid'] == 10) {
+    $element['#title'] = 'My custom title';
+    $element['#default_value'] = 42;
+  }
+}
+
+/**
+ * Performs the conditional action set on an implemented component.
+ *
+ * Setting the form element allows form validation functions to see the value
+ * that webform has set for the given component.
+ *
+ * @param array $component
+ *   The webform component array whose value is being set for the currently-
+ *   edited submission.
+ * @param array $element
+ *   The form element currently being set.
+ * @param array $form_state
+ *   The form's state.
+ * @param string $value
+ *   The value to be set, as defined in the conditional action.
+ */
+function _webform_action_set_component($component, &$element, &$form_state, $value) {
+  $element['#value'] = $value;
+  form_set_value($element, $value, $form_state);
+}
+
 /**
  * A hook for changing the input values before saving to the database.
  *
@@ -738,6 +1047,7 @@ function _webform_display_component($component, $value, $format = 'html') {
  *   A Webform component array.
  * @param $value
  *   The POST data associated with the user input.
+ *
  * @return
  *   An array of values to be saved into the database. Note that this should be
  *   a numerically keyed array.
@@ -745,8 +1055,7 @@ function _webform_display_component($component, $value, $format = 'html') {
 function _webform_submit_component($component, $value) {
   // Clean up a phone number into 123-456-7890 format.
   if ($component['extra']['phone_number']) {
-    $matches = array();
-    $number = preg_replace('[^0-9]', $value[0]);
+    $number = preg_replace('/[^0-9]/', '', $value[0]);
     if (strlen($number) == 7) {
       $number = substr($number, 0, 3) . '-' . substr($number, 3, 4);
     }
@@ -770,10 +1079,9 @@ function _webform_submit_component($component, $value) {
  */
 function _webform_delete_component($component, $value) {
   // Delete corresponding files when a submission is deleted.
-  $filedata = unserialize($value['0']);
-  if (isset($filedata['filepath']) && is_file($filedata['filepath'])) {
-    unlink($filedata['filepath']);
-    db_query("DELETE FROM {files} WHERE filepath = '%s'", $filedata['filepath']);
+  if (!empty($value[0]) && ($file = webform_get_file($value[0]))) {
+    file_usage_delete($file, 'webform');
+    file_delete($file);
   }
 }
 
@@ -827,11 +1135,31 @@ function _webform_theme_component() {
  *   Boolean flag determining if the details about a single component are being
  *   shown. May be used to provided detailed information about a single
  *   component's analysis, such as showing "Other" options within a select list.
- * @return
- *   An array of data rows, each containing a statistic for this component's
- *   submissions.
+ * @param $join
+ *   An optional SelectQuery object to be used to join with the submissions
+ *   table to restrict the submissions being analyzed.
+ *
+ * @return array
+ *   An array containing one or more of the following keys:
+ *   - table_rows: If this component has numeric data that can be represented in
+ *     a grid, return the values here. This array assumes a 2-dimensional
+ *     structure, with the first value being a label and subsequent values
+ *     containing a decimal or integer.
+ *   - table_header: If this component has more than a single set of values,
+ *     include a table header so each column can be labeled.
+ *   - other_data: If your component has non-numeric data to include, such as
+ *     a description or link, include that in the other_data array. Each item
+ *     may be a string or an array of values that matches the number of columns
+ *     in the table_header property.
+ *   At the very least, either table_rows or other_data should be provided.
+ *   Note that if you want your component's analysis to be available by default
+ *   without the user specifically enabling it, you must set
+ *   $component['extra']['analysis'] = TRUE in your
+ *   _webform_defaults_component() callback.
+ *
+ * @see _webform_defaults_component()
  */
-function _webform_analysis_component($component, $sids = array(), $single = FALSE) {
+function _webform_analysis_component($component, $sids = array(), $single = FALSE, $join = NULL) {
   // Generate the list of options and questions.
   $options = _webform_select_options_from_text($component['extra']['options'], TRUE);
   $questions = _webform_select_options_from_text($component['extra']['questions'], TRUE);
@@ -850,6 +1178,10 @@ function _webform_analysis_component($component, $sids = array(), $single = FALS
     $query->condition('sid', $sids, 'IN');
   }
 
+  if ($join) {
+    $query->innerJoin($join, 'ws2_', 'wsd.sid = ws2_.sid');
+  }
+
   $result = $query->execute();
   $counts = array();
   foreach ($result as $data) {
@@ -873,9 +1205,15 @@ function _webform_analysis_component($component, $sids = array(), $single = FALS
     }
     $rows[] = $row;
   }
-  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('class' => array('webform-grid'))));
 
-  return array(array(array('data' => $output, 'colspan' => 2)));
+  $other = array();
+  $other[] = l(t('More information'), 'node/' . $component['nid'] . '/webform-results/analysis/' . $component['cid']);
+
+  return array(
+    'table_header' => $header,
+    'table_rows' => $rows,
+    'other_data' => $other,
+  );
 }
 
 /**
@@ -889,7 +1227,8 @@ function _webform_analysis_component($component, $sids = array(), $single = FALS
  * @param $value
  *   An array of information containing the submission result, directly
  *   correlating to the webform_submitted_data database schema.
- * @return
+ *
+ * @return string
  *   Textual output formatted for human reading.
  */
 function _webform_table_component($component, $value) {
@@ -919,14 +1258,15 @@ function _webform_table_component($component, $value) {
  *   A Webform component array.
  * @param $export_options
  *   An array of options that may configure export of this field.
- * @return
+ *
+ * @return array
  *   An array of data to be displayed in the first three rows of a CSV file, not
  *   including either prefixed or trailing commas.
  */
 function _webform_csv_headers_component($component, $export_options) {
   $header = array();
   $header[0] = array('');
-  $header[1] = array($component['name']);
+  $header[1] = array($export_options['header_keys'] ? $component['form_key'] : $component['name']);
   $items = _webform_component_options($component['extra']['questions']);
   $count = 0;
   foreach ($items as $key => $item) {
@@ -956,7 +1296,8 @@ function _webform_csv_headers_component($component, $export_options) {
  * @param $value
  *   An array of information containing the submission result, directly
  *   correlating to the webform_submitted_data database schema.
- * @return
+ *
+ * @return array
  *   An array of items to be added to the CSV file. Each value within the array
  *   will be another column within the file. This function is called once for
  *   every row of data.
@@ -970,6 +1311,107 @@ function _webform_csv_data_component($component, $export_options, $value) {
   return $return;
 }
 
+/**
+ * Adjusts the view field(s) that are automatically generated for number
+ * components.
+ *
+ * Provides each component the opportunity to adjust how this component is
+ * displayed in a view as a field in a view table. For example, a component may
+ * modify how it responds to click-sorting. Or it may add additional fields,
+ * such as a grid component having a column for each question.
+ *
+ * @param array $component
+ *   A Webform component array.
+ * @param array $fields
+ *   An array of field-definition arrays. Will be passed one field definition,
+ *   which may be modified. Additional fields may be added to the array.
+ *
+ * @return array
+ *   The modified $fields array.
+ */
+function _webform_view_field_component($component, $fields) {
+  foreach ($fields as &$field) {
+    $field['webform_datatype'] = 'number';
+  }
+  return $fields;
+}
+
+/**
+ * Modify the how a view was expanded to show all the components.
+ *
+ * This alter function is only called when the view is actually modified. It
+ * provides modules an opportunity to alter the changes that webform made to
+ * the view.
+ *
+ * This hook is called from webform_views_pre_view. If another module also
+ * changes views by implementing this same views hook, the relative order of
+ * execution of the two implementations will depend upon the module weights of
+ * the two modules. Using hook_webform_view_alter instead guarantees an
+ * opportunity to modify the view AFTER webform.
+ *
+ * @param object $view
+ *   The view object.
+ * @param string $display_id
+ *   The display_id that was expanded by webform.
+ * @param array $args
+ *   The arguments that were passed to the view.
+ */
+function hook_webform_view_alter($view, $display_id, $args) {
+  // Don't show component with cid == 4.
+  $fields = $view->get_items('field', $display_id);
+  foreach ($fields as $id => $field) {
+    if (isset($field['webform_cid']) && $field['webform_cid'] == 4) {
+      unset($fields[$id]);
+    }
+  }
+  $view->display[$display_id]->handler->set_option('fields', $fields);
+}
+
+/**
+ * Modify the list of mail systems that are capable of sending HTML email.
+ *
+ * @param array &$systems
+ *   An array of mail system class names.
+ */
+function hook_webform_html_capable_mail_systems_alter(&$systems) {
+  if (module_exists('my_module')) {
+    $systems[] = 'MyModuleMailSystem';
+  }
+}
+
+/**
+ * Define a list of webform exporters.
+ *
+ * @return array
+ *   A list of the available exporters provided by the module.
+ *
+ * @see webform_webform_exporters()
+ */
+function hook_webform_exporters() {
+  $exporters = array(
+    'webform_exporter_custom' => array(
+      'title' => t('Webform exporter name'),
+      'description' => t('The description for this exporter.'),
+      'handler' => 'webform_exporter_custom',
+      'file' => drupal_get_path('module', 'yourmodule') . '/includes/webform_exporter_custom.inc',
+      'weight' => 10,
+    ),
+  );
+
+  return $exporters;
+}
+
+/**
+ * Modify the list of webform exporters definitions.
+ *
+ * @param array &$exporters
+ *   A list of all available webform exporters.
+ */
+function hook_webform_exporters_alter(&$exporters) {
+  $exporters['excel']['handler'] = 'customized_excel_exporter';
+  $exporters['excel']['file'] = drupal_get_path('module', 'yourmodule') . '/includes/customized_excel_exporter.inc';
+}
+
 /**
  * @}
  */
